/*
 * Copyright (C) 2024 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include "level_zero/sysman/source/shared/windows/product_helper/sysman_product_helper_hw.h"

namespace L0 {
namespace Sysman {

static EnableSysmanProductHelper<IGFX_ALDERLAKE_P> enableAdlp;

} // namespace Sysman
} // namespace L0
