/*
 * Copyright (C) 2022-2024 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once

#include <vector>

namespace NEO {
const std::vector<unsigned short> ehlDeviceIds{
    0x4500,
    0x4541,
    0x4551,
    0x4571,
    0x4555,
    0x4E51,
    0x4E61,
    0x4E71,
    0x4E55};
} // namespace NEO
