/*
 * Copyright (C) 2022 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once

#include "shared/source/os_interface/os_inc_base.h"

namespace Os {

extern const char *igaDllName;

} // namespace Os