/*
 * Copyright (C) 2022-2024 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once

#include <vector>

namespace NEO {
const std::vector<unsigned short> icllpDeviceIds{
    0xFF05,
    0x8A56,
    0x8A58,
    0x8A5C,
    0x8A5A,
    0x8A50,
    0x8A52,
    0x8A51};
} // namespace NEO
