/*
 * Copyright (C) 2024 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once

#ifndef _XE_DRM_H_
#include <linux/types.h>
#include <sys/ioctl.h>
namespace NEO {
namespace XeDrm {
#include "xe_drm.h"
}
} // namespace NEO
using namespace NEO::XeDrm;

#endif
