/*
 * Copyright (C) 2022-2024 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once

#include <vector>

namespace NEO {
const std::vector<unsigned short> adlnDeviceIds{0x46D0, 0x46D1, 0x46D2, 0x46D3, 0x46D4};
} // namespace NEO
