/*
 * Copyright (C) 2022 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once
#include "ze_barrier_api_entrypoints.h"
#include "ze_cmdlist_api_entrypoints.h"
#include "ze_cmdqueue_api_entrypoints.h"
#include "ze_context_api_entrypoints.h"
#include "ze_copy_api_entrypoints.h"
#include "ze_device_api_entrypoints.h"
#include "ze_driver_api_entrypoints.h"
#include "ze_event_api_entrypoints.h"
#include "ze_fence_api_entrypoints.h"
#include "ze_image_api_entrypoints.h"
#include "ze_memory_api_entrypoints.h"
#include "ze_module_api_entrypoints.h"
#include "ze_sampler_api_entrypoints.h"