/*
 * Copyright (C) 2019-2020 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include "opencl/source/mem_obj/image.h"

namespace NEO {

template <typename GfxFamily>
void ImageHw<GfxFamily>::appendSurfaceStateExt(void *memory) {
}

} // namespace NEO
