/*
 * Copyright (C) 2023-2024 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once

namespace NEO {

enum class ReleaseType {
    release1255,
    release1256,
    release1257,
    release1260,
    release1261,
    release1270,
    release1271,
    release1274,
    release2001,
    release2004,
};

} // namespace NEO
