/*
 * Copyright (C) 2020-2023 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include "shared/source/gen9/hw_cmds.h"
#include "shared/source/image/image_surface_state.h"

namespace NEO {

typedef Gen9Family Family;

#include "shared/source/image/image_skl_and_later.inl"
} // namespace NEO
