/*
 * Copyright (C) 2019-2022 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include "shared/source/gen12lp/hw_cmds_base.h"
using Family = NEO::Gen12LpFamily;
constexpr static auto gfxCore = IGFX_GEN12LP_CORE;
#include "opencl/source/sampler/sampler_tgllp_and_later.inl"
