/*
 * Copyright (C) 2019-2023 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once

#include "shared/test/common/tests_configuration.h"

namespace NEO {
inline constexpr TestMode defaultTestMode = TestMode::unitTests;
} // namespace NEO
