/*
 * Copyright (C) 2020 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once

#include "shared/source/compiler_interface/compiler_cache.h"

namespace NEO {
CompilerCacheConfig getDefaultCompilerCacheConfig();
}
