/*
 * Copyright (C) 2019-2021 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once

#define PATH_SEPARATOR '/'
#define __cdecl
namespace Os {
// Pci Path
extern const char *sysFsPciPathPrefix;
extern const char *pciDevicesDirectory;
// Proc Path
extern const char *sysFsProcPathPrefix;
} // namespace Os
