/*
 * Copyright (C) 2022-2024 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once

#include <vector>

namespace NEO {
const std::vector<unsigned short> bdwDeviceIds{
    0x1602,
    0x160A,
    0x1606,
    0x160E,
    0x160D,
    0x1612,
    0x161A,
    0x1616,
    0x161E,
    0x161D,
    0x1622,
    0x162A,
    0x1626,
    0x162B,
    0x162E,
    0x162D};
} // namespace NEO
