/*
 * Copyright (C) 2024 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once

namespace NEO {

#define CREATE_LOGGER(...)

#define LOG_CRITICAL(...)
#define LOG_CRITICAL_FOR_CORE(...)

#define LOG_INFO(...)

extern void initLogger();

} // namespace NEO
