/*
 * Copyright (C) 2021-2023 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include "shared/source/debug_settings/debug_settings_manager.h"

namespace NEO {
DebugSettingsManager<globalDebugFunctionalityLevel> debugManager("");
}
