/*
 * Copyright (C) 2022-2024 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once

#include <vector>

namespace NEO {
const std::vector<unsigned short> rklDeviceIds{
    0x4C80,
    0x4C8A,
    0x4C8B,
    0x4C8C,
    0x4C90,
    0x4C9A};
} // namespace NEO
