/*
 * Copyright (C) 2023 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once

namespace NEO {

class ApiGfxCoreHelper {
  public:
    ApiGfxCoreHelper() = default;
    virtual ~ApiGfxCoreHelper() = default;
};

} // namespace NEO
