/*
 * Copyright (C) 2018-2021 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include "shared/source/os_interface/os_interface.h"

#include "shared/source/gmm_helper/gmm_interface.h"

namespace NEO {

bool OSInterface::osEnableLocalMemory = true;

} // namespace NEO
