/*
 * Copyright (C) 2024 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include "shared/source/xe2_hpg_core/hw_cmds_lnl.h"

namespace NEO {

void LNL::adjustHardwareInfo(HardwareInfo *hwInfo) {
}

} // namespace NEO
