/*
 * Copyright (C) 2020-2021 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once
namespace L0 {
namespace ult {

template <typename Type>
struct Mock : public Type {};

} // namespace ult
} // namespace L0
