/*
 * Copyright (C) 2020-2021 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include <cstdint>
// above is workaround for copyright checker issue

namespace Os {
const char *openglDllName = "mock_opengl32.dll";
}
