/*
 * Copyright (C) 2022-2024 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once

#include <vector>

namespace NEO {
const std::vector<unsigned short> tgllpDeviceIds{
    0x9A49,
    0x9A40,
    0x9A59,
    0x9A60,
    0x9A68,
    0x9A70,
    0x9A78};
} // namespace NEO
