/*
 * Copyright (C) 2021 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include "shared/source/command_container/implicit_scaling.h"

namespace NEO {
namespace ImplicitScaling {
bool apiSupport = true;
} // namespace ImplicitScaling
} // namespace NEO
