/*
 * Copyright (C) 2022-2024 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include "shared/source/execution_environment/execution_environment.h"

namespace NEO {

void ExecutionEnvironment::adjustRootDeviceEnvironments() {
}

void ExecutionEnvironment::configureCcsMode() {
}

void ExecutionEnvironment::restoreCcsMode() {
}

} // namespace NEO
