/*
 * Copyright (C) 2018-2020 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include "opencl/test/unit_test/api/cl_enqueue_migrate_mem_objects_tests.inl"
#include "opencl/test/unit_test/api/cl_enqueue_native_kernel_tests.inl"
#include "opencl/test/unit_test/api/cl_enqueue_nd_range_kernel_tests.inl"
#include "opencl/test/unit_test/api/cl_enqueue_read_buffer_rect_tests.inl"
#include "opencl/test/unit_test/api/cl_enqueue_read_buffer_tests.inl"
#include "opencl/test/unit_test/api/cl_enqueue_read_image_tests.inl"
#include "opencl/test/unit_test/api/cl_enqueue_svm_free_tests.inl"
#include "opencl/test/unit_test/api/cl_enqueue_svm_map_tests.inl"
#include "opencl/test/unit_test/api/cl_enqueue_svm_mem_fill_tests.inl"
#include "opencl/test/unit_test/api/cl_enqueue_svm_memcpy_tests.inl"
#include "opencl/test/unit_test/api/cl_enqueue_svm_unmap_tests.inl"
#include "opencl/test/unit_test/api/cl_enqueue_task_tests.inl"
#include "opencl/test/unit_test/api/cl_enqueue_unmap_mem_object_tests.inl"
#include "opencl/test/unit_test/api/cl_enqueue_verify_memory.inl"
#include "opencl/test/unit_test/api/cl_enqueue_wait_for_events_tests.inl"
#include "opencl/test/unit_test/api/cl_enqueue_write_buffer_rect_tests.inl"
#include "opencl/test/unit_test/api/cl_enqueue_write_buffer_tests.inl"
#include "opencl/test/unit_test/api/cl_enqueue_write_image_tests.inl"
#include "opencl/test/unit_test/api/cl_finish_tests.inl"
#include "opencl/test/unit_test/api/cl_flush_tests.inl"
#include "opencl/test/unit_test/api/cl_get_context_info_tests.inl"
#include "opencl/test/unit_test/api/cl_get_device_and_host_timer.inl"
#include "opencl/test/unit_test/api/cl_get_device_ids_tests.inl"
#include "opencl/test/unit_test/api/cl_get_device_info_tests.inl"
#include "opencl/test/unit_test/api/cl_get_event_profiling_info_tests.inl"
#include "opencl/test/unit_test/api/cl_get_extension_function_address_for_platform_tests.inl"
#include "opencl/test/unit_test/api/cl_get_extension_function_address_tests.inl"
#include "opencl/test/unit_test/api/cl_get_image_info_tests.inl"
#include "opencl/test/unit_test/api/cl_get_image_params_tests.inl"
#include "opencl/test/unit_test/api/cl_get_kernel_arg_info_tests.inl"
#include "opencl/test/unit_test/api/cl_get_kernel_info_tests.inl"
#include "opencl/test/unit_test/api/cl_get_kernel_sub_group_info_khr_tests.inl"
#include "opencl/test/unit_test/api/cl_get_kernel_sub_group_info_tests.inl"
