/*
 * Copyright (C) 2023 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include "level_zero/core/source/rtas/rtas.h"

namespace L0 {
std::string RTASBuilder::rtasLibraryName = "ze_intel_gpu_raytracing.dll";
} // namespace L0
