/*
 * Copyright (C) 2024 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include "shared/source/xe2_hpg_core/hw_cmds_base.h"
using Family = NEO::Xe2HpgCoreFamily;
constexpr static auto gfxCore = IGFX_XE2_HPG_CORE;
#include "opencl/source/sampler/sampler_tgllp_and_later.inl"
