/*
 * Copyright (C) 2019-2021 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once
#ifdef SUPPORT_ICLLP
#include "hw_info_icllp.h"
#endif
#ifdef SUPPORT_LKF
#include "hw_info_lkf.h"
#endif
#ifdef SUPPORT_EHL
#include "hw_info_ehl.h"
#endif
