/*
 * Copyright (C) 2020-2021 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include "shared/source/helpers/deferred_deleter_helper.h"

bool NEO::isDeferredDeleterEnabled() {
    return false;
} // namespace NEO
