/*
 * Copyright (C) 2021 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include <string>

namespace NEO {

bool isCompatibleDriverStore(std::string &&deviceRegistryPath) {
    return false;
}

} // namespace NEO