/*
 * Copyright (C) 2019-2023 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once

#include <cstdint>

namespace GrfConfig {
inline constexpr uint32_t defaultGrfNumber = 128u;
inline constexpr uint32_t largeGrfNumber = 256u;
inline constexpr uint32_t notApplicable = 0u;
} // namespace GrfConfig
