/*
 * Copyright (C) 2024 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once

#ifdef SUPPORT_BMG
#include "hw_info_bmg.h"
#endif

#ifdef SUPPORT_LNL
#include "hw_info_lnl.h"
#endif
