/*
 * Copyright (C) 2021 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include "shared/source/os_interface/linux/os_library_linux.h"

namespace NEO {
namespace Linux {
void adjustLibraryFlags(int &dlopenFlag) {
}
} // namespace Linux
} // namespace NEO
