/*
 * Copyright (C) 2024 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include "shared/source/compiler_interface/oclc_extensions_extra.h"

namespace NEO {
void getOpenclCFeaturesListExtra(const ReleaseHelper *releaseHelper, OpenClCFeaturesContainer &openclCFeatures) {
}
} // namespace NEO