/*
 * Copyright (C) 2019-2020 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include "opencl/source/os_interface/ocl_reg_path.h"

namespace NEO {
const char *oclRegPath = "Software\\Intel\\IGFX\\OCL\\";
}
