/*
 * Copyright (C) 2022-2023 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once
#include "platforms.h"

namespace AOT_PVC {
inline constexpr AOT::PRODUCT_CONFIG productConfigs[] = {
    AOT::PVC_XL_A0,
    AOT::PVC_XL_A0P,
    AOT::PVC_XT_A0,
    AOT::PVC_XT_B0,
    AOT::PVC_XT_B1,
    AOT::PVC_XT_C0,
    AOT::PVC_XT_C0_VG};
}
