/*
 * Copyright (C) 2023 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

namespace L0 {
namespace ult {
const char *testLoaderDllName = "test_l0_loader_lib.dll";
const char *testDllName = "test_dynamic_lib.dll";
} // namespace ult
} // namespace L0