/*
 * Copyright (C) 2024 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once

#include <vector>

namespace NEO {
const std::vector<unsigned short> bmgG21DeviceIds{0xE202, 0xE20B, 0xE20C, 0xE20D, 0xE212};
} // namespace NEO
