/*
 * Copyright (C) 2023-2024 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once
#include "platforms.h"

namespace AOT_ARL {
inline constexpr AOT::PRODUCT_CONFIG productConfigs[] = {
    AOT::MTL_U_B0,
    AOT::ARL_H_A0,
    AOT::ARL_H_B0};
}
