/*
 * Copyright (C) 2018-2021 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include "opencl/source/helpers/base_object.h"

namespace NEO {

std::thread::id invalidThreadID;

} // namespace NEO
