/*
 * Copyright (C) 2022-2024 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once

#include <vector>

namespace NEO {
const std::vector<unsigned short> pvcXlDeviceIds{0x0BD0};
const std::vector<unsigned short> pvcXtDeviceIds{0x0BD5, 0x0BD6, 0x0BD7, 0x0BD8, 0x0BD9, 0x0BDA, 0x0BDB, 0x0B69, 0x0B6E};
const std::vector<unsigned short> pvcXtVgDeviceIds{0x0BD4};
} // namespace NEO
