/*
 * Copyright (C) 2024 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include "opencl/test/unit_test/device/device_caps_test_utils.h"

namespace NEO {
void verifyAnyRemainingOpenclCFeatures(const ReleaseHelper *releaseHelper, const cl_name_version *&iterator) {
}
} // namespace NEO