/*
 * Copyright (C) 2021-2024 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once
#include "shared/test/common/gen11/test_traits_gen11.h"
#include "shared/test/common/gen12lp/test_traits_gen12lp.h"
#include "shared/test/common/gen8/test_traits_gen8.h"
#include "shared/test/common/gen9/test_traits_gen9.h"
#include "shared/test/common/xe2_hpg_core/test_traits_xe2_hpg_core.h"
#include "shared/test/common/xe_hpc_core/test_traits_xe_hpc_core.h"
#include "shared/test/common/xe_hpg_core/test_traits_xe_hpg_core.h"
