/*
 * Copyright (C) 2023 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include "shared/source/compiler_interface/compiler_options_extra.h"

namespace NEO {
namespace CompilerOptions {
void applyExtraInternalOptions(std::string &internalOptions, const CompilerProductHelper &compilerProductHelper) {
}

} // namespace CompilerOptions
} // namespace NEO
