/*
 * Copyright (C) 2020-2022 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once

#include <stdint.h>

namespace NEO {
int32_t getNewKernelArbitrationPolicy(uint32_t policy);
} // namespace NEO
