/*
 * Copyright (C) 2023 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once

namespace L0 {
namespace ult {
extern const char *testLoaderDllName;
extern const char *testDllName;
} // namespace ult
} // namespace L0