/*
 * Copyright (C) 2018-2021 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

__kernel void test(__global float *argGlobal, __read_only image3d_t argImg3D, __constant float *argConst) {
}
