/*
 * Copyright (C) 2021 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include "shared/test/common/helpers/ult_hw_config.h"

namespace NEO {
UltHwConfig ultHwConfig{};
} // namespace NEO
