/*
 * Copyright (C) 2020 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include "opencl/test/unit_test/mt_tests/api/cl_create_user_event_tests_mt.inl"
#include "opencl/test/unit_test/mt_tests/api/cl_get_device_info_tests_mt.inl"
#include "opencl/test/unit_test/mt_tests/api/cl_get_platform_ids_tests_mt.inl"
#include "opencl/test/unit_test/mt_tests/api/cl_get_platform_info_tests_mt.inl"
#include "opencl/test/unit_test/mt_tests/api/cl_intel_tracing_tests_mt.inl"
#include "opencl/test/unit_test/mt_tests/api/cl_set_mem_object_destructor_callback_tests_mt.inl"
