/*
 * Copyright (C) 2018-2021 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include "shared/source/gen8/hw_info_gen8.h"

namespace NEO {
const char *GfxFamilyMapper<IGFX_GEN8_CORE>::name = "Gen8";
} // namespace NEO
