/*
 * Copyright (C) 2024 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once

#include <vector>

namespace NEO {
const std::vector<unsigned short> lnlDeviceIds{0x6420, 0x64A0, 0x64B0};
} // namespace NEO
