/*
 * Copyright (C) 2021 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include "shared/source/direct_submission/direct_submission_controller.h"

namespace NEO {
bool DirectSubmissionController::isSupported() {
    return true;
}
} // namespace NEO