/*
 * Copyright (C) 2022-2023 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

using namespace NEO;

constexpr static auto gfxProduct = IGFX_BROADWELL;
#include "shared/source/os_interface/product_helper_hw.inl"