/*
 * Copyright (C) 2021-2023 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once

#include "shared/source/xe_hpc_core/hw_cmds_xe_hpc_core_base.h"

#ifdef SUPPORT_PVC
#include "hw_cmds_pvc.h"
#endif
