/*
 * Copyright (C) 2020 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once

namespace NEO {

// Method called by global factory enabler
template <typename Type>
void populateFactoryTable();

} // namespace NEO
