/*
 * Copyright (C) 2022-2024 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once

#include <linux/types.h>
#include <sys/ioctl.h>
namespace NEO {
namespace PrelimI915 {
#include "i915_drm_prelim.h"
}
} // namespace NEO
using namespace NEO::PrelimI915;
