/*
 * Copyright (C) 2018-2020 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include "opencl/test/unit_test/mocks/mock_async_event_handler.h"
namespace MockAsyncEventHandlerGlobals {
bool destructorCalled = false;
}