/*
 * Copyright (C) 2018-2021 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once

namespace NEO {

namespace D3DTypesHelper {
struct D3D9 {
};
struct D3D10 {
};
struct D3D11 {
};
} // namespace D3DTypesHelper

template <typename D3D>
class D3DSharingFunctions {
};
} // namespace NEO
