/*
 * Copyright (C) 2022-2024 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once

#include <vector>

namespace NEO {
const std::vector<unsigned short> bxtDeviceIds{
    0x9906,
    0x9907,
    0x0A84,
    0x5A84,
    0x5A85,
    0x1A85,
    0x1A84,
    0x9908};
} // namespace NEO
