/*
 * Copyright (C) 2019-2022 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once
#include <cstdint>

namespace NEO {
inline constexpr uint32_t maxRootDeviceCount = 3u;
} // namespace NEO
