/*
 * Copyright (C) 2022-2023 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include "level_zero/sysman/source/shared/firmware_util/sysman_firmware_util_imp.h"

namespace L0 {
namespace Sysman {
std::string FirmwareUtilImp::fwUtilLibraryName = "igsc.dll";
} // namespace Sysman
} // namespace L0