/*
 * Copyright (C) 2023-2024 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include "shared/source/os_interface/windows/wddm/wddm.h"

namespace NEO {

void Wddm::setProcessPowerThrottling() {}

void Wddm::setThreadPriority() {}

} // namespace NEO