/*
 * Copyright (C) 2019-2021 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once
#include <cstdint>

namespace NEO {
namespace EngineLimits {

constexpr uint32_t maxHandleCount = 4u;

}; // namespace EngineLimits
} // namespace NEO
