/*
 * Copyright (C) 2022 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include "shared/test/common/mocks/mock_os_library.h"

NEO::OsLibrary *MockOsLibrary::loadLibraryNewObject = nullptr;
