/*
 * Copyright (C) 2024 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include "shared/test/unit_test/os_interface/linux/xe/mock_drm_xe.h"

#include "shared/test/unit_test/os_interface/linux/xe/mock_drm_xe_definitions.inl"
