/*
 * Copyright (C) 2020-2024 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include "shared/test/common/helpers/kernel_binary_helper.h"

const std::string KernelBinaryHelper::BUILT_INS("15672580764041246108");
const std::string KernelBinaryHelper::BUILT_INS_WITH_IMAGES("8269212628108939982_images");
