/*
 * Copyright (C) 2021-2023 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include "shared/source/gen9/hw_cmds_skl.h"
#include "shared/source/os_interface/product_helper.h"
#include "shared/source/os_interface/product_helper.inl"
#include "shared/source/os_interface/product_helper_bdw_and_later.inl"

constexpr static auto gfxProduct = IGFX_SKYLAKE;

#include "shared/source/gen9/skl/os_agnostic_product_helper_skl.inl"
#include "shared/source/os_interface/product_helper_before_gen12lp.inl"

template class NEO::ProductHelperHw<gfxProduct>;
