/*
 * Copyright (C) 2021 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once

#ifndef NEO_DISABLE_SOURCE_FILE_PATHS
#define NEO_SOURCE_FILE_PATH __FILE__
#else
#define NEO_SOURCE_FILE_PATH ""
#endif
