/*
 * Copyright (C) 2021 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once
#include <string>

namespace NEO {
extern std::string getRevision();
extern std::string getOclDriverVersion();
} // namespace NEO
