/*
 * Copyright (C) 2020 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include "shared/source/debug_settings/debug_variables_helper.h"

namespace NEO {

bool isDebugKeysReadEnabled() {
    return false;
}

} // namespace NEO
