/*
 * Copyright (C) 2022-2024 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once

#include <vector>

namespace NEO {
const std::vector<unsigned short> adlpDeviceIds{
    0x46A0,
    0x46B0,
    0x46A1,
    0x46A3,
    0x46A6,
    0x46A8,
    0x46AA,
    0x462A,
    0x4626,
    0x4628,
    0x46B1,
    0x46B3,
    0x46C0,
    0x46C1,
    0x46C3,
    0xA7A0,
    0xA720,
    0xA7A8,
    0xA7A1,
    0xA721,
    0xA7A9,
    0xA7AA,
    0xA7AB,
    0xA7AC,
    0xA7AD};
} // namespace NEO
