/*
 * Copyright (C) 2020-2023 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once
#include "shared/source/built_ins/built_in_ops_base.h"

namespace NEO {
namespace EBuiltInOps {
inline constexpr Type maxCoreValue{maxBaseValue};
}
} // namespace NEO
