/*
 * Copyright (C) 2023 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once
#include "cstdint"

namespace NEO {

namespace TimestampPacketConstants {
static constexpr uint32_t preferredPacketCount = 16;
static constexpr uint32_t initValue = 1;
} // namespace TimestampPacketConstants

} // namespace NEO