/*
 * Copyright (C) 2018-2024 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once
#include "shared/source/gmm_helper/gmm_lib.h"

#include "GL/gl.h"
#include "third_party/opencl_headers/CL/cl_gl.h"

// header with structures shared with opengl
#include "opencl/extensions/public/cl_gl_private_intel_structures.h"
