/*
 * Copyright (C) 2022-2024 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once

#include <vector>

namespace NEO {
const std::vector<unsigned short> dg2G10DeviceIds{
    0x4F80,
    0x4F81,
    0x4F82,
    0x4F83,
    0x4F84,
    0x5690,
    0x5691,
    0x5692,
    0x56A0,
    0x56A1,
    0x56A2,
    0x56C0,
    0x56C2,
    0x56BE,
    0x56BF};

const std::vector<unsigned short> dg2G11DeviceIds{
    0x4F87,
    0x4F88,
    0x5693,
    0x5694,
    0x5695,
    0x56A5,
    0x56A6,
    0x56B0,
    0x56B1,
    0x56BA,
    0x56BB,
    0x56BC,
    0x56BD,
    0x56C1};

const std::vector<unsigned short> dg2G12DeviceIds{
    0x5696,
    0x5697,
    0x56A3,
    0x56A4,
    0x56B2,
    0x56B3,
    0x4F85,
    0x4F86};
} // namespace NEO
