/*
 * Copyright (C) 2021 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include "shared/source/xe_hpg_core/hw_info_xe_hpg_core.h"

namespace NEO {
const char *GfxFamilyMapper<IGFX_XE_HPG_CORE>::name = "XE_HPG_CORE";
} // namespace NEO
