/*
 * Copyright (C) 2021 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once
#include "shared/source/command_container/definitions/encode_surface_state_args_base.h"

namespace NEO {

struct EncodeSurfaceStateArgs : EncodeSurfaceStateArgsBase {
    EncodeSurfaceStateArgs() = default;
};
} // namespace NEO
