/*
 * Copyright (C) 2019-2021 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include "opencl/source/mem_obj/buffer.h"

#include "encode_surface_state_args.h"

namespace NEO {

void Buffer::appendSurfaceStateArgs(EncodeSurfaceStateArgs &args) {
}

} // namespace NEO
