/*
 * Copyright (C) 2021-2023 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once

#ifdef SUPPORT_DG2
#include "hw_info_dg2.h"
#endif

#ifdef SUPPORT_MTL
#include "hw_info_mtl.h"
#endif

#ifdef SUPPORT_ARL
#include "hw_info_arl.h"
#endif
