/*
 * Copyright (C) 2020-2021 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include "level_zero/core/source/compiler_interface/l0_reg_path.h"

namespace L0 {
const char *registryPath = "Software\\Intel\\IGFX\\L0\\";
}
