/*
 * Copyright (C) 2022-2024 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once

#include <vector>

namespace NEO {
const std::vector<unsigned short> amlDeviceIds{0x591C};
const std::vector<unsigned short> kblDeviceIds{
    0x5902,
    0x590B,
    0x590A,
    0x5906,
    0x590E,
    0x5908,
    0x5913,
    0x5915,
    0x5912,
    0x591B,
    0x5917,
    0x591A,
    0x5916,
    0x591E,
    0x591D,
    0x5921,
    0x5926,
    0x5927,
    0x592B,
    0x592A,
    0x5923,
    0x5932,
    0x593B,
    0x593A,
    0x593D};
} // namespace NEO
