/*
 * Copyright (C) 2021 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include "shared/test/common/helpers/default_hw_info.h"

namespace NEO {

std::unique_ptr<HardwareInfo> defaultHwInfo;
} // namespace NEO
