/*
 * Copyright (C) 2022-2024 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once

#include <vector>

namespace NEO {
const std::vector<unsigned short> dg1DeviceIds{
    0x4905,
    0x4906,
    0x4907,
    0x4908,
    0x4909};
} // namespace NEO
