/*
 * Copyright (C) 2021-2022 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#include "shared/source/os_interface/linux/system_info.h"

namespace NEO {

void SystemInfo::checkSysInfoMismatch(HardwareInfo *hwInfo) {}

} // namespace NEO
