/*
 * Copyright (C) 2022-2024 Intel Corporation
 *
 * SPDX-License-Identifier: MIT
 *
 */

#pragma once

#include <vector>

namespace NEO {
const std::vector<unsigned short> adlsDeviceIds{
    0x4680,
    0x4682,
    0x4688,
    0x468A,
    0x468B,
    0x4690,
    0x4692,
    0x4693,
    0xA780,
    0xA781,
    0xA782,
    0xA783,
    0xA788,
    0xA789,
    0xA78A,
    0xA78B};

} // namespace NEO
